#!/usr/bin/python

# aprswatch by wa5znu
# Copyright 2010-2011 Leigh L. Klotz, Jr. See file COPYRIGHT
#
# given a list of callsign-id on the command line, send mail when 
# they send packets.  send no more than one message every N minutes
# per callsign-id per waypoint or untagged location.

import sys
import socket
import smtplib
from email.mime.text import MIMEText
from datetime import datetime
import time
from packet import Packet
from geocode import Geocode
from waypoints import Waypoints
from config import Config

LAST_PACKET_TIME={}
LAST_VOLTAGE={}
LAST_VOLTAGE_TIME={}

waypoints = Waypoints()

def logprint(msg):
    now = datetime.now()
    print(now.strftime("%Y-%m-%dT%H:%M:%S") + ": " + msg)

def errprint(err):
    now = datetime.now()
    print(now.strftime("%Y-%m-%dT%H:%M:%S") + ":")
    print(err)

    

# todo: prune ancient LAST_PACKET_TIMES when new packet seen to prevent
# uncontrolled growth
# todo: sometimes socket gives us infinite empty lines; we need to check
# that and exit.

def doit(calls):
    logprint("looking for %s" % (calls))
    while True:
        send_admin_note("Opening connection to %s:%s" % (Config.APRS_IS_HOST, Config.APRS_IS_PORT))
        try:
            sSock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sSock.connect((Config.APRS_IS_HOST, Config.APRS_IS_PORT))
        except socket.error, msg:
            send_admin_note("Error opening socket: %s.  Sleeping for 60 seconds." % (msg))
            time.sleep(60)
            continue
        send_admin_note("Connected to %s:%s" % (Config.APRS_IS_HOST, Config.APRS_IS_PORT))
        sent = sSock.send("user %s pass %s vers %s\r\n" % (Config.APRS_IS_CALL, Config.APRS_IS_PASSCODE, Config.VERSION))
        if (sent == 0):
            raise RuntimeError("Failed to log in")
        send_admin_note("Logged in as %s on %s:%s" % (Config.APRS_IS_CALL, Config.APRS_IS_HOST, Config.APRS_IS_PORT))        
        empty_line_count = 0
        while empty_line_count < Config.MAX_EMPTY_LINES:
            line = sSock.recv(Config.BUFSIZE).strip()
            if line == "":
                empty_line_count = empty_line_count + 1
            else:
                empty_line_count = 0
                process_packet(line, calls)
        send_admin_note("Empty line count exceeded; closing connection and sleeping for 60 seconds.")
        sSock.close()
        time.sleep(60)
        
def process_packet(line, calls):
    logprint(line)
    p = Packet(line.rstrip('\r\n'))
    #logprint("line " + line)
    #logprint("data " + p.getData())
    #send_admin_note(p.getData())
    call = p.getCall().upper()
    if call != '' and ((call in calls) or ('*' in calls)):
        now = datetime.now()
        try:
            loc = packet_geocode(p)
            colloquial = waypoints.lookup(loc)
            if colloquial:
              message = line + "\n" + colloquial + "\n" + loc
            else:
              message = line + "\n" + loc              
        except:
            colloquial = None
            message = line
        try:
            battery_voltage = packet_voltage(p)
            if battery_voltage != None:
                battery_condition = voltage_check(packet_vtable(p), battery_voltage, call)
                if battery_condition != None:
                    sendnote(Config.BATTERY_FROM, Config.BATTERY_TO, Config.BATTERY_SUBJECT % (call, battery_condition, battery_voltage), call, Config.BATTERY_SUBJECT % (call, battery_condition, battery_voltage))
        except Exception as ee:
            logprint("battery_voltage failed: %s" % (p.getComment()))
            errprint(ee)
        if colloquial:
          subject = Config.MOTION_AT_SUBJECT % (call, colloquial)
        else:
          subject = Config.MOTION_SUBJECT % (call)
        maybe_send_motion_note(subject, call, colloquial, message)

def packet_geocode(p):
    lat = p.getLat()
    lon = p.getLon()
    if lat != 0 and lon != 0:
        return Geocode().getlocation(lat, lon)
    else:
        return ""
    

def packet_voltage(p):
    battery_voltage = p.getComment().strip();
    if (battery_voltage.endswith("V")):
        return float(battery_voltage[:-1])
    else:
        return None

def packet_vtable(p):
    s = p.getSym() + p.getSym2()
    table = Config.VOLTAGE_CHECKS.get(s)
    if (table == None):
        return {}
    else:
        return table

def voltage_check(vtable, battery_voltage, call):
    now = datetime.now()
    if LAST_VOLTAGE.has_key(call) and LAST_VOLTAGE[call] < battery_voltage:
        for test_voltage,test_label in sorted(vtable.items()):
            if battery_voltage <= test_voltage:
                if check_time_delta(now, call, LAST_VOLTAGE_TIME):
                    return test_label
                else:
                    logprint("Not sending note about %s %s voltage %gV because it's not time" % (call, test_label, battery_voltage))
                    return None
    LAST_VOLTAGE[call] = battery_voltage
    return None
    

# todo will send note about non-named location immediately after each named location
def maybe_send_motion_note(subject, call, colloquial, line):
    now = datetime.now()
    if colloquial:
      key = call + "@" + colloquial
    else:
      key = call
    if check_time_delta(now, key, LAST_PACKET_TIME):
        sendnote(Config.MOTION_FROM, Config.MOTION_TO, subject, call, line)
    else:
        logprint("Not sending note about %s@%s because it's not time" % (call, colloquial))

def sendnote(from_addr, to_addr, subject, call, line):
    logprint("Sending mail: " + subject)
    # Create a text/plain message
    msg = MIMEText(Config.MOTION_BODY % (call, call, line))

    # me == the sender's email address
    # you == the recipient's email address
    msg['Subject'] = subject
    msg['From'] = from_addr
    msg['To'] = to_addr

    s = smtplib.SMTP(Config.MAIL_HOST)
    s.sendmail(from_addr, [to_addr], msg.as_string())

def send_admin_note(body):
    logprint("APRS Admin: " + body)
    # Create a text/plain message
    msg = MIMEText(body)

    # me == the sender's email address
    # you == the recipient's email address
    msg['Subject'] = "APRS Admin"
    msg['From'] = Config.ADMIN_FROM
    msg['To'] = Config.ADMIN_TO

    s = smtplib.SMTP(Config.MAIL_HOST)
    s.sendmail(Config.ADMIN_FROM, [Config.ADMIN_TO], msg.as_string())

def check_time_delta(now, key, timetable):
    if (key not in timetable):
        timetable[key] = now
        return True
    else:
        last_time = timetable[key]
        delta = (now) - (last_time)
        if delta > Config.TIME_DELTA_MINUTES:
            timetable[key] = now
            return True
        else:
            return False

if __name__=='__main__':
    doit([ call.upper() for call in sys.argv[1:] ])
